==> emodpy_tbhiv-0.0.6-py3-none-any.whl: emodpy_tbhiv/demographics/TBHIVDemographics.py <==
from emod_api.demographics.Demographics import Demographics, Node

class TBHIVDemographics(Demographics):
    """
    This class is derived from emod_api.demographics' Demographics class so that we can set 
    certain defaults for TBHIV in construction. Keen observers will note thatt SetDefaultProperties
    does not look like a TBHIV-specific function. But as we add other disease types the 
    generatlizations and speicfics will become clearer. The architectural point is solid.
    """
    def __init__(self, nodes, idref="Gridded world grump2.5arcmin", base_file=None, implicits=None): 
        super().__init__( nodes, idref, base_file, implicits )
        super().SetDefaultNodeAttributes()
        self.SetHIVCoInfectionDistribution()
        self.SetHIVTBCoInfectionMortalityDistribution()

    def SetHIVCoInfectionDistribution( self ):
        """
        Insert some notion of a default HIVCoInfection distribution.
        """
        default_hivco = { "HIVCoinfectionDistribution":
            {
                "NumDistributionAxes": 3,
                "AxisNames": [ "gender", "time", "age" ],
                "AxisUnits": [ "male =0, female=1", "years", "years" ],
                "AxisScaleFactors": [ 1, 365, 365 ],
                "NumPopulationGroups": [ 2, 3, 4 ],
                "PopulationGroups":
                [
                    [ 0, 1 ],
                    [ 0, 50, 100],
                    [ 0, 15, 35, 50]
                ],
                "ResultUnits": "Probability of HIV infection per repetition of HIV Outbreak",
                "ResultScaleFactor": 0.019199999999999998,
                "ResultValues":
                [

                    [
                        [1, 1, 1, 1],
                        [1, 1, 1, 1],
                        [1, 1, 1, 1]

                    ],

                     [
                        [1, 1, 1, 1],
                        [1, 1, 1, 1],
                        [1, 1, 1, 1]

                    ]


                ]
            }
        }
        self.raw['Defaults']['IndividualAttributes'].update( 
            default_hivco 
        )

    def SetHIVTBCoInfectionMortalityDistribution( self ):
        """
        Insert some notion of a default HIVTBCoInfection Mortality distribution.
        """
        default_setting = {
            "HIVTBCoinfMortalityDistribution": {
                "AxisNames": [
                    "age",
                    "year"
                ],
                "AxisScaleFactors": [
                    365,
                    1
                ],
                "AxisUnits": [
                    "years",
                    "years"
                ],
                "NumDistributionAxes": 2,
                "NumPopulationGroups": [
                    2,
                    1
                ],
                "PopulationGroups": [
                    [
                        0,
                        15
                    ],
                    [
                        0
                    ]
                ],
                "ResultScaleFactor": 2.739726027397e-06,
                "ResultUnits": "annual deaths per 1000 individuals",
                "ResultValues": [
                    [
                        14
                    ],
                    [
                        14
                    ]
                ]
            }
        }
        self.raw['Defaults']['IndividualAttributes'].update( 
            default_setting 
        )


def fromBasicNode(lat=0, lon=0, pop=1e6, name=1, forced_id=1, implicit_config_fns = None ):
    """
    This function creates a single-node TBHIVDemographics instance from the params you give it. 
    """
    new_nodes = [ Node(lat=lat, lon=lon, pop=pop, name=name, forced_id=forced_id) ]
    return TBHIVDemographics(nodes=new_nodes, implicits=implicit_config_fns)

def fromData(): 
    import pandas as pd
    import numpy as np
    import seaborn as sns
    from collections import OrderedDict
    from functools import reduce
    import json
    #import matplotlib.pyplot as plt
    from sklearn.linear_model import LinearRegression

    name_conversion_dict = {'Age (x)': 'Age',
                            'Central death rate m(x,n)': 'Mortality_mid',
                            'Age interval (n)': 'Interval',
                            'Period': 'Years'
                            }

    sex_dict = {'Male': 0, 'Female': 1}

    def min_not_nan(x_list):
        loc_in = list(filter(lambda x: not np.isnan(x), x_list))
        return np.min(loc_in)

    def map_year(x_tuple, flag='mid'):

        valid_entries_loc = ['mid', 'end', 'start']

        if flag not in valid_entries_loc:
            raise ValueError('invalid endpoint specified')

        if flag == 'mid':
            return (x_tuple[0] + x_tuple[1]) / 2.0
        elif flag == 'start':
            return x_tuple[0]
        else:
            return x_tuple[1]

    def construct_interval(x, y):
        return (x, x + y)

    def midpoint(x, y):
        return (x + y) / 2.0

    def generate_dict_order(tuple_list, which_entry=1):

        my_unordered_list = tuple_list.apply(lambda x: x[which_entry])
        dict_to_order = OrderedDict(zip(tuple_list, my_unordered_list))

        return dict_to_order

    def __init__(self, file_male,file_female, regression_interval=[1970, 1980],which_point = 'mid', predict_horizon=2050):
        self.filenames = [file_male, file_female]
        self.interval_fit = regression_interval
        self.predict_horizon = predict_horizon
        self.estimates_list = []
        self.which_point = which_point

    def do_extrapolation(self, add_to_list=True):
        df_mort_male = pd.read_csv(self.filenames[0], usecols=Demog.name_conversion_dict)
        df_mort_male['Sex'] = 'Male'
        df_mort_female = pd.read_csv(self.filenames[1], usecols=Demog.name_conversion_dict)
        df_mort_female['Sex'] = 'Female'
        df_mort = pd.concat([df_mort_male, df_mort_female], axis=0)
        df_mort.rename(columns=Demog.name_conversion_dict, inplace=True)
        df_mort['Years'] = df_mort['Years'].apply(lambda xx: tuple(
            [float(zz) for zz in xx.split('-')]))  # this might be a bit too format specific (ie dashes in input)

        # log transform the data and drop unneeded columns
        df_mort['log_Mortality_mid'] = df_mort['Mortality_mid'].apply(lambda x: np.log(x))
        df_mort['Age'] = df_mort[['Age', 'Interval']].apply(lambda zz: Demog.construct_interval(*zz), axis=1)

        year_order_dict = Demog.generate_dict_order(df_mort['Years'])
        age_order_dict = Demog.generate_dict_order(df_mort['Age'])
        df_mort['sortby2'] = df_mort['Age'].map(age_order_dict)
        df_mort['sortby1'] = df_mort['Sex'].map(Demog.sex_dict)
        df_mort['sortby3'] = df_mort['Years'].map(year_order_dict)
        df_mort.sort_values(['sortby1', 'sortby2', 'sortby3'], inplace=True)
        df_mort.drop(columns=['Mortality_mid', 'Interval', 'sortby1', 'sortby2', 'sortby3'], inplace=True)

        # convert to years (and to string for age_list due to really annoying practical slicing reasons
        df_mort['Years'] = df_mort['Years'].apply(lambda x: Demog.map_year(x, self.which_point))
        df_mort['Age'] = df_mort['Age'].apply(lambda x: str(x))
        df_before_time = df_mort[df_mort['Years'].between(0, self.interval_fit[0])].copy()

        df_mort.set_index(['Sex', 'Age'], inplace=True)
        sex_list = list(set(df_mort.index.get_level_values('Sex')))
        age_list = list(set(df_mort.index.get_level_values('Age')))

        df_list = []
        df_list_future = []
        for sex in sex_list:
            for age in age_list:
                tmp_data = df_mort.loc[(sex, age, slice(None)), :]
                extrap_model = LinearRegression(normalize=True, fit_intercept=True, copy_X=True)
                extrap_model_2 = LinearRegression(normalize=True, fit_intercept=True, copy_X=True)

                first_extrap_df = tmp_data[tmp_data['Years'].between(self.interval_fit[0], self.interval_fit[1])]
                XX = tmp_data[tmp_data['Years'].between(self.interval_fit[0], self.predict_horizon)].values[:, 0]

                values = first_extrap_df.values
                extrap_model.fit(values[:, 0].reshape(-1, 1), values[:, 1])

                extrap_predictions = extrap_model.predict(XX.reshape(-1, 1))

                loc_df = pd.DataFrame.from_dict({'Sex': sex, 'Age': age, 'Years': XX, 'Extrap': extrap_predictions})
                loc_df.set_index(['Sex', 'Age', 'Years'], inplace=True)

                df_list.append(loc_df.copy())

        df_e1 = pd.concat(df_list, axis=0)

        df_list_final = [df_mort, df_e1]
        df_total = reduce(lambda left, right: pd.merge(left, right, on=['Sex', 'Age', 'Years']), df_list_final)

        df_total = df_total.reset_index(inplace=False).set_index(['Sex', 'Age'], inplace=False)

        df_total['Extrap'] = df_total['Extrap'].apply(np.exp)
        df_total['Data'] = df_total['log_Mortality_mid'].apply(np.exp)
        df_before_time['Data'] = df_before_time['log_Mortality_mid'].apply(np.exp)

        df_before_time.set_index(['Sex', 'Age'], inplace=True)
        df_total = pd.concat([df_total, df_before_time], axis=0, join='outer', sort=True)

        df_total.reset_index(inplace=True)
        df_total['sortby2'] = df_total['Age'].map(age_order_dict)
        df_total['sortby1'] = df_total['Sex'].map(Demog.sex_dict)
        df_total.sort_values(by=['sortby1', 'sortby2', 'Years'], inplace=True)
        df_total.drop(columns=['sortby1', 'sortby2'], inplace=True)

        if not add_to_list:
            self.estimates_list = [df_total.copy()]
        else:
            self.estimates_list.append(df_total.copy())

    """
    def makeplots(self, save=None):

        for i,df_loc in enumerate(self.estimates_list):

            df_loc.reset_index(inplace=True)

            grid = sns.FacetGrid(df_loc, col='Age', row='Sex')
            grid.map(plt.plot, 'Years', 'Extrap', color='b', label='Pre-epidemic')
            grid.map(plt.plot, 'Years', 'Data', color='r', linestyle='--', label='Data', marker='None')
            df_loc['Min_Mortality'] = df_loc[['Data', 'Extrap']].apply(Demog.min_not_nan, axis=1)

            if save:
                plt.savefig(str(save) + '_' + str(i) + '.png')
            else:
                plt.show()
    """

    def create_json_overlay(self, template, output_name='Extract_demog.json', csv_out=False, n=0, results_scale_factor=1.0/365.0):

        df = self.estimates_list[n]
        df['FE'] = df[['Data', 'Extrap']].apply(Demog.min_not_nan, axis=1)
        df['Age'] = df['Age'].apply(lambda x: int(x.split(',')[1].split(')')[0]))
        male_df = df[df['Sex'] == 'Male']
        female_df = df[df['Sex'] == 'Female']

        male_df.set_index(['Sex','Age', 'Years'], inplace=True)
        female_df.set_index(['Sex','Age','Years'], inplace=True)
        male_data = male_df['FE']
        female_data = female_df['FE']

        male_data = male_data.unstack(-1)
        male_data.sort_index(level ='Age', inplace=True)
        female_data = female_data.unstack(-1)
        female_data.sort_index(level='Age', inplace=True)

        years_out_male = list(male_data.columns)
        years_out_female =list( female_data.columns)

        age_out_male = list(male_data.index.get_level_values('Age'))
        age_out_female = list(male_data.index.get_level_values('Age'))

        male_output= male_data.values
        female_output = female_data.values

        if csv_out:
            male_data.to_csv('Male' + csv_out)
            female_data.to_csv('Female' + csv_out)

        with open(template,'r') as f:
            out_json = json.load(f)

        dict_female = {'NumPopulationGroups': list(female_data.shape),
                       'AxisNames': ['age', 'year'],
                       'AxisScaleFactors': [365.0, 1],
                       'AxisUnits': ['years', 'years'],
                       'NumDistributionAxes': 2,
                       'PopulationGroups': [age_out_female, years_out_female],
                       'ResultsScaleFactor': results_scale_factor,
                       'ResultsUnits': 'annual deaths per capita',
                       'ResultsValues': female_output.tolist()
                       }

        dict_male =   {'NumPopulationGroups': list(male_data.shape),
                       'AxisNames': ['age', 'year'],
                       'AxisScaleFactors': [365.0, 1],
                       'AxisUnits': ['years', 'years'],
                       'NumDistributionAxes': 2,
                       'PopulationGroups': [age_out_male, years_out_male],
                       'ResultsScaleFactor': results_scale_factor,
                       'ResultsUnits': 'annual deaths per capita',
                       'ResultsValues': male_output.tolist()
                       }
        out_json['Defaults']['IndividualAttributes']['MortalityDistributionFemale'] = dict_female
        out_json['Defaults']['IndividualAttributes']['MortalityDistributionMale'] = dict_male

        out_data = json.dumps(out_json, indent=4)

        with open(output_name,'w') as f_out:
            f_out.write(out_data)

    # placeholders for command line arguments
    make_plots = True

    # User variables where to start extrapolation (ie before HIV epidemic) and how far to look back
    # All in years unless noted otherwise
    extrap_point = 1980
    look_back = 20
    time_horizon = 2100
    now_ish = 1980
    look_back_from_now_ish = 20

    filename_male = 'Malawi_male_mortality.csv'
    filename_female = 'Malawi_female_mortality.csv'

    interval_for_fitting_regression = [1970, 1980]
    predict_horizons = 2050

    #This is all you need to use here (Note you prob want to override the default on extrapolation
    myob = Demog(filename_male, filename_female) #creatrs object
    myob.do_extrapolation()  #does extrapolation (if you do more than one it adds it to a list so you can compare)
    myob.makeplots(save=False) # make plots by age
    myob.create_json_overlay('template.json',output_name='Output_demog.json', csv_out='Test.csv') #create json

==> emodpy_tbhiv-0.0.14-py3-none-any.whl: emodpy_tbhiv/demographics/TBHIVDemographics.py <==
import pandas as pd
import numpy as np
import json

from emod_api.demographics.Demographics import Demographics, Node

class TBHIVDemographics(Demographics):
    """
    This class is derived from emod_api.demographics' Demographics class so that we can set 
    certain defaults for TBHIV in construction. Keen observers will note thatt SetDefaultProperties
    does not look like a TBHIV-specific function. But as we add other disease types the 
    generatlizations and speicfics will become clearer. The architectural point is solid.
    """
    def __init__(self, nodes=None, idref="Gridded world grump2.5arcmin", base_file=None):
        if nodes is None:
            nodes = [ Node(pop=pop, lat=0, lon=0 ) ]
        super().__init__( nodes, idref, base_file )
        super().SetDefaultNodeAttributes()
        self.SetHIVCoInfectionDistribution()
        self.SetHIVTBCoInfectionMortalityDistribution()


    def SetHIVCoInfectionDistribution( self ):
        """
        Insert some notion of a default HIVCoInfection distribution.
        """
        default_hivco = { "HIVCoinfectionDistribution":
            {
                "NumDistributionAxes": 3,
                "AxisNames": [ "gender", "time", "age" ],
                "AxisUnits": [ "male =0, female=1", "years", "years" ],
                "AxisScaleFactors": [ 1, 365, 365 ],
                "NumPopulationGroups": [ 2, 3, 4 ],
                "PopulationGroups":
                [
                    [ 0, 1 ],
                    [ 0, 50, 100],
                    [ 0, 15, 35, 50]
                ],
                "ResultUnits": "Probability of HIV infection per repetition of HIV Outbreak",
                "ResultScaleFactor": 0.019199999999999998,
                "ResultValues":
                [

                    [
                        [1, 1, 1, 1],
                        [1, 1, 1, 1],
                        [1, 1, 1, 1]

                    ],

                     [
                        [1, 1, 1, 1],
                        [1, 1, 1, 1],
                        [1, 1, 1, 1]

                    ]


                ]
            }
        }
        self.raw['Defaults']['IndividualAttributes'].update( 
            default_hivco 
        )

    def SetHIVTBCoInfectionMortalityDistribution( self ):
        """
        Insert some notion of a default HIVTBCoInfection Mortality distribution.
        """
        default_setting = {
            "HIVTBCoinfMortalityDistribution": {
                "AxisNames": [
                    "age",
                    "year"
                ],
                "AxisScaleFactors": [
                    365,
                    1
                ],
                "AxisUnits": [
                    "years",
                    "years"
                ],
                "NumDistributionAxes": 2,
                "NumPopulationGroups": [
                    2,
                    1
                ],
                "PopulationGroups": [
                    [
                        0,
                        15
                    ],
                    [
                        0
                    ]
                ],
                "ResultScaleFactor": 2.739726027397e-06,
                "ResultUnits": "annual deaths per 1000 individuals",
                "ResultValues": [
                    [
                        14
                    ],
                    [
                        14
                    ]
                ]
            }
        }
        self.raw['Defaults']['IndividualAttributes'].update( 
            default_setting 
        )

class TBHIVDemographicsWithGenderExtrapolation(TBHIVDemographics):
    def __init__(self, file_male,file_female, pop=1e6, regression_interval=[1970, 1980],which_point = 'mid', predict_horizon=2050):
        new_nodes = [ Node(pop=pop, lat=0, lon=0, forced_id=1 ) ]
        super().__init__( nodes=new_nodes  )
        self.filenames = [file_male, file_female]
        self.interval_fit = regression_interval
        self.predict_horizon = predict_horizon
        self.estimates_list = []
        self.which_point = which_point

    def do_extrapolation(self, add_to_list=True):
        from collections import OrderedDict
        from sklearn.linear_model import LinearRegression
        from functools import reduce

        name_conversion_dict = {'Age (x)': 'Age',
                                'Central death rate m(x,n)': 'Mortality_mid',
                                'Age interval (n)': 'Interval',
                                'Period': 'Years'
                                }
        sex_dict = {'Male': 0, 'Female': 1}

        def construct_interval(x, y):
            return (x, x + y)

        def midpoint(x, y):
            return (x + y) / 2.0

        def generate_dict_order(tuple_list, which_entry=1):
            my_unordered_list = tuple_list.apply(lambda x: x[which_entry])
            dict_to_order = OrderedDict(zip(tuple_list, my_unordered_list))
            return dict_to_order

        def map_year(x_tuple, flag='mid'):
            valid_entries_loc = ['mid', 'end', 'start']

            if flag not in valid_entries_loc:
                raise ValueError('invalid endpoint specified')

            if flag == 'mid':
                return (x_tuple[0] + x_tuple[1]) / 2.0
            elif flag == 'start':
                return x_tuple[0]
            else:
                return x_tuple[1]


        df_mort_male = pd.read_csv(self.filenames[0], usecols=name_conversion_dict)
        df_mort_male['Sex'] = 'Male'
        df_mort_female = pd.read_csv(self.filenames[1], usecols=name_conversion_dict)
        df_mort_female['Sex'] = 'Female'
        df_mort = pd.concat([df_mort_male, df_mort_female], axis=0)
        df_mort.rename(columns=name_conversion_dict, inplace=True)
        df_mort['Years'] = df_mort['Years'].apply(lambda xx: tuple(
            [float(zz) for zz in xx.split('-')]))  # this might be a bit too format specific (ie dashes in input)

        # log transform the data and drop unneeded columns
        df_mort['log_Mortality_mid'] = df_mort['Mortality_mid'].apply(lambda x: np.log(x))
        df_mort['Age'] = df_mort[['Age', 'Interval']].apply(lambda zz: construct_interval(*zz), axis=1)

        year_order_dict = generate_dict_order(df_mort['Years'])
        age_order_dict = generate_dict_order(df_mort['Age'])
        df_mort['sortby2'] = df_mort['Age'].map(age_order_dict)
        df_mort['sortby1'] = df_mort['Sex'].map(sex_dict)
        df_mort['sortby3'] = df_mort['Years'].map(year_order_dict)
        df_mort.sort_values(['sortby1', 'sortby2', 'sortby3'], inplace=True)
        df_mort.drop(columns=['Mortality_mid', 'Interval', 'sortby1', 'sortby2', 'sortby3'], inplace=True)

        # convert to years (and to string for age_list due to really annoying practical slicing reasons
        df_mort['Years'] = df_mort['Years'].apply(lambda x: map_year(x, self.which_point))
        df_mort['Age'] = df_mort['Age'].apply(lambda x: str(x))
        df_before_time = df_mort[df_mort['Years'].between(0, self.interval_fit[0])].copy()

        df_mort.set_index(['Sex', 'Age'], inplace=True)
        sex_list = list(set(df_mort.index.get_level_values('Sex')))
        age_list = list(set(df_mort.index.get_level_values('Age')))

        df_list = []
        df_list_future = []
        for sex in sex_list:
            for age in age_list:
                tmp_data = df_mort.loc[(sex, age, slice(None)), :]
                extrap_model = LinearRegression(normalize=True, fit_intercept=True, copy_X=True)
                extrap_model_2 = LinearRegression(normalize=True, fit_intercept=True, copy_X=True)

                first_extrap_df = tmp_data[tmp_data['Years'].between(self.interval_fit[0], self.interval_fit[1])]
                XX = tmp_data[tmp_data['Years'].between(self.interval_fit[0], self.predict_horizon)].values[:, 0]

                values = first_extrap_df.values
                extrap_model.fit(values[:, 0].reshape(-1, 1), values[:, 1])

                extrap_predictions = extrap_model.predict(XX.reshape(-1, 1))

                loc_df = pd.DataFrame.from_dict({'Sex': sex, 'Age': age, 'Years': XX, 'Extrap': extrap_predictions})
                loc_df.set_index(['Sex', 'Age', 'Years'], inplace=True)

                df_list.append(loc_df.copy())

        df_e1 = pd.concat(df_list, axis=0)

        df_list_final = [df_mort, df_e1]
        df_total = reduce(lambda left, right: pd.merge(left, right, on=['Sex', 'Age', 'Years']), df_list_final)

        df_total = df_total.reset_index(inplace=False).set_index(['Sex', 'Age'], inplace=False)

        df_total['Extrap'] = df_total['Extrap'].apply(np.exp)
        df_total['Data'] = df_total['log_Mortality_mid'].apply(np.exp)
        df_before_time['Data'] = df_before_time['log_Mortality_mid'].apply(np.exp)

        df_before_time.set_index(['Sex', 'Age'], inplace=True)
        df_total = pd.concat([df_total, df_before_time], axis=0, join='outer', sort=True)

        df_total.reset_index(inplace=True)
        df_total['sortby2'] = df_total['Age'].map(age_order_dict)
        df_total['sortby1'] = df_total['Sex'].map(sex_dict)
        df_total.sort_values(by=['sortby1', 'sortby2', 'Years'], inplace=True)
        df_total.drop(columns=['sortby1', 'sortby2'], inplace=True)

        if not add_to_list:
            self.estimates_list = [df_total.copy()]
        else:
            self.estimates_list.append(df_total.copy())

    def makeplots(self, save=None):
        pass
        """
        import seaborn as sns

        for i,df_loc in enumerate(self.estimates_list):

            df_loc.reset_index(inplace=True)

            grid = sns.FacetGrid(df_loc, col='Age', row='Sex')
            grid.map(plt.plot, 'Years', 'Extrap', color='b', label='Pre-epidemic')
            grid.map(plt.plot, 'Years', 'Data', color='r', linestyle='--', label='Data', marker='None')
            df_loc['Min_Mortality'] = df_loc[['Data', 'Extrap']].apply(Demog.min_not_nan, axis=1)

            if save:
                plt.savefig(str(save) + '_' + str(i) + '.png')
            else:
                plt.show()
        """

    def create_json_overlay(self, template, output_name='Extract_demog.json', csv_out=False, n=0, results_scale_factor=1.0/365.0):
        """
        This almost certainly needs to be refactored to use the base class 'generate_file' functionality.
        """

        def min_not_nan(x_list):
            loc_in = list(filter(lambda x: not np.isnan(x), x_list))
            return np.min(loc_in)

        df = self.estimates_list[n]
        df['FE'] = df[['Data', 'Extrap']].apply(min_not_nan, axis=1)
        df['Age'] = df['Age'].apply(lambda x: int(x.split(',')[1].split(')')[0]))
        male_df = df[df['Sex'] == 'Male']
        female_df = df[df['Sex'] == 'Female']

        male_df.set_index(['Sex','Age', 'Years'], inplace=True)
        female_df.set_index(['Sex','Age','Years'], inplace=True)
        male_data = male_df['FE']
        female_data = female_df['FE']

        male_data = male_data.unstack(-1)
        male_data.sort_index(level ='Age', inplace=True)
        female_data = female_data.unstack(-1)
        female_data.sort_index(level='Age', inplace=True)

        years_out_male = list(male_data.columns)
        years_out_female =list( female_data.columns)

        age_out_male = list(male_data.index.get_level_values('Age'))
        age_out_female = list(male_data.index.get_level_values('Age'))

        male_output= male_data.values
        female_output = female_data.values

        if csv_out:
            male_data.to_csv('Male' + csv_out)
            female_data.to_csv('Female' + csv_out)

        # TBD: This is the part that should use base file functionality
        
        dict_female = {'NumPopulationGroups': list(female_data.shape),
                       'AxisNames': ['age', 'year'],
                       'AxisScaleFactors': [365.0, 1],
                       'AxisUnits': ['years', 'years'],
                       'NumDistributionAxes': 2,
                       'PopulationGroups': [age_out_female, years_out_female],
                       'ResultsScaleFactor': results_scale_factor,
                       'ResultsUnits': 'annual deaths per capita',
                       'ResultsValues': female_output.tolist()
                       }

        dict_male =   {'NumPopulationGroups': list(male_data.shape),
                       'AxisNames': ['age', 'year'],
                       'AxisScaleFactors': [365.0, 1],
                       'AxisUnits': ['years', 'years'],
                       'NumDistributionAxes': 2,
                       'PopulationGroups': [age_out_male, years_out_male],
                       'ResultsScaleFactor': results_scale_factor,
                       'ResultsUnits': 'annual deaths per capita',
                       'ResultsValues': male_output.tolist()
                       }
        male_mort = { "MortalityDistributionMale" : dict_male }
        female_mort = { "MortalityDistributionFemale" : dict_female }
        self.SetDefaultFromTemplate( male_mort )
        self.SetDefaultFromTemplate( female_mort )
        self.generate_file( output_name )

#
# 'STATIC' instantiator functions that are really module functions
#

def fromBasicNode(lat=0, lon=0, pop=1e6, name=1, forced_id=1, implicit_config_fns = None ):
    """
    This function creates a single-node TBHIVDemographics instance from the params you give it. 
    """
    new_nodes = [ Node(lat=lat, lon=lon, pop=pop, name=name, forced_id=forced_id) ]
    return TBHIVDemographics(nodes=new_nodes)

def fromData( pop=1e6, filename_male = 'Malawi_male_mortality.csv', filename_female = 'Malawi_female_mortality.csv' ):
    #import matplotlib.pyplot as plt
    # placeholders for command line arguments
    make_plots = True

    # User variables where to start extrapolation (ie before HIV epidemic) and how far to look back
    # All in years unless noted otherwise
    extrap_point = 1980
    look_back = 20
    time_horizon = 2100
    now_ish = 1980
    look_back_from_now_ish = 20


    interval_for_fitting_regression = [1970, 1980]
    predict_horizons = 2050

    #This is all you need to use here (Note you prob want to override the default on extrapolation
    myob = TBHIVDemographicsWithGenderExtrapolation(filename_male, filename_female,pop=pop) #creatrs object
    myob.do_extrapolation()  #does extrapolation (if you do more than one it adds it to a list so you can compare)
    myob.makeplots(save=False) # make plots by age
    myob.create_json_overlay('template.json',output_name='Output_demog.json', csv_out='Test.csv') #create json
    return myob
